/**
 * @file tdd_pixel_ws2812_opt.c
 * @author www.tuya.com
 * @brief tdd_pixel_ws2812 module is used to
 * @version 0.1
 * @date 2022-08-03
 *
 * @copyright Copyright (c) tuya.inc 2022
 *
 */

/*********************************************************************
********************************includes******************************
*********************************************************************/
#include "uni_log.h"
#include "tkl_spi.h"
#include "tdl_pixel_driver.h"
#include "tdd_pixel_basic.h"

#include "tdd_pixel_ws2812_opt.h"
/*********************************************************************
******************************macro define****************************
*********************************************************************/
/* SPI波特率 */
#define DRV_SPI_SPEED 2887500

#define COLOR_PRIMARY_NUM 3
#define COLOR_RESOLUTION  255

// V2.0归零码4bit版
#define LED_DRVICE_IC_DATA_00 0X88 // 00
#define LED_DRVICE_IC_DATA_01 0X8e // 01
#define LED_DRVICE_IC_DATA_10 0Xe8 // 10
#define LED_DRVICE_IC_DATA_11 0Xee // 11

#define ONE_BYTE_LEN_4BIT  4
#define PIXEL_CHIP_SPI_NUM 0
/*********************************************************************
****************************typedef define****************************
*********************************************************************/

/*********************************************************************
****************************variable define***************************
*********************************************************************/
STATIC RGB_ORDER_MODE_E rgb_order_mode = RGB_ORDER;
/*********************************************************************
****************************function define***************************
*********************************************************************/
/**
 * @brief       rgb转成spi数据 4bit为单位
 *
 * @param[in]   color_data          颜色数据
 * @param[in]   chip_ic_0           0码
 * @param[in]   chip_ic_1           1码
 * @param[out]  spi_data_buf        转化后的spi数据
 *
 * @return none
 */
STATIC void __tdd_2812_4bit_rgb_transform_spi_data(unsigned char color_data, unsigned char *spi_data_buf)
{
    unsigned char i = 0;

    for (i = 0; i < ONE_BYTE_LEN_4BIT; i++) {
        if ((color_data & 0xc0) == 0) {
            spi_data_buf[i] = LED_DRVICE_IC_DATA_00;

        } else if ((color_data & 0xc0) == 0x40) {
            spi_data_buf[i] = LED_DRVICE_IC_DATA_01;

        } else if ((color_data & 0xc0) == 0x80) {
            spi_data_buf[i] = LED_DRVICE_IC_DATA_10;

        } else if ((color_data & 0xc0) == 0xc0) {
            spi_data_buf[i] = LED_DRVICE_IC_DATA_11;

        } else {
            PR_ERR("SPI Send 1/0 Bit error\r\n");
        }

        color_data <= 2;
    }

    return;
}

/**
 * @function:__tdd_2812_driver_open
 * @brief: 打开（初始化）设备
 * @param[in]: pixel_num -> 像素点数
 * @param[out]: *handle  -> 设备句柄
 * @return: success -> 0  fail -> else
 */
STATIC OPERATE_RET __tdd_2812_driver_open(OUT DRIVER_HANDLE_T *handle, IN unsigned short pixel_num)
{
    OPERATE_RET op_ret = OPRT_OK;
    TUYA_SPI_BASE_CFG_T spi_cfg = {0};
    DRV_PIXEL_TX_CTRL_T *pixels_send = NULL;
    unsigned int tx_buf_len = 0;

    if (NULL == handle || (0 == pixel_num)) {
        return OPRT_INVALID_PARM;
    }
    extern void tkl_spi_set_spic_flag(void);
    tkl_spi_set_spic_flag();
    spi_cfg.role = TUYA_SPI_ROLE_MASTER;
    spi_cfg.mode = TUYA_SPI_MODE0;
    spi_cfg.type = TUYA_SPI_SOFT_TYPE;
    spi_cfg.databits = TUYA_SPI_DATA_BIT8;
    spi_cfg.freq_hz = DRV_SPI_SPEED;
    op_ret = tkl_spi_init(PIXEL_CHIP_SPI_NUM, &spi_cfg);
    if (op_ret != OPRT_OK) {
        PR_ERR("tkl_spi_init fail op_ret:%d", op_ret);
        return op_ret;
    }

    tx_buf_len = ONE_BYTE_LEN_4BIT * COLOR_PRIMARY_NUM * pixel_num;
    op_ret = tdd_pixel_create_tx_ctrl(tx_buf_len, &pixels_send);
    if (op_ret != OPRT_OK) {
        return op_ret;
    }

    *handle = pixels_send;

    return OPRT_OK;
}

/**
 * @function: __tdd_ws2812_driver_send_data
 * @brief: 将颜色数据（RGBCW）转换为当前芯片的线序并转换为SPI码流, 通过SPI发送
 * @param[in]: handle -> 设备句柄
 * @param[in]: *data_buf -> 颜色数据
 * @param[in]: buf_len -> 颜色数据长度
 * @return: success -> 0  fail -> else
 */
STATIC OPERATE_RET __tdd_ws2812_driver_send_data(IN DRIVER_HANDLE_T handle, IN unsigned short *data_buf,
                                                 IN unsigned int buf_len)
{
    OPERATE_RET ret = OPRT_OK;
    DRV_PIXEL_TX_CTRL_T *tx_ctrl = NULL;
    unsigned short swap_buf[COLOR_PRIMARY_NUM] = {0};
    unsigned int i = 0, j = 0, idx = 0;

    if (NULL == handle || NULL == data_buf || 0 == buf_len) {
        return OPRT_INVALID_PARM;
    }

    tx_ctrl = (DRV_PIXEL_TX_CTRL_T *)handle;
    for (j = 0; j < buf_len / COLOR_PRIMARY_NUM; j++) {
        memset(swap_buf, 0, sizeof(swap_buf));
        tdd_rgb_line_seq_transform(&data_buf[j * COLOR_PRIMARY_NUM], swap_buf, rgb_order_mode);
        for (i = 0; i < COLOR_PRIMARY_NUM; i++) {
            __tdd_2812_4bit_rgb_transform_spi_data((unsigned char)swap_buf[i], &tx_ctrl->tx_buffer[idx]);
            idx += ONE_BYTE_LEN_4BIT;
        }
    }

    ret = tkl_spi_send(PIXEL_CHIP_SPI_NUM, tx_ctrl->tx_buffer, tx_ctrl->tx_buffer_len);

    return ret;
}

/**
 * @function: __tdd_ws2812_driver_close
 * @brief: 关闭设备（资源释放）
 * @param[in]: *handle -> 设备句柄
 * @return: success -> 0  fail -> else
 */
STATIC OPERATE_RET __tdd_ws2812_driver_close(IN DRIVER_HANDLE_T *handle)
{
    DRV_PIXEL_TX_CTRL_T *tx_ctrl = NULL;

    if ((NULL == handle) || (*handle == NULL)) {
        return OPRT_INVALID_PARM;
    }

    tx_ctrl = (DRV_PIXEL_TX_CTRL_T *)(*handle);

    tkl_spi_deinit(PIXEL_CHIP_SPI_NUM);
    tdd_pixel_tx_ctrl_release(tx_ctrl);
    *handle = NULL;

    return OPRT_OK;
}

/**
 * @function:tdd_ws2812_driver_register
 * @brief: 注册设备
 * @param[in]: *driver_name -> 设备名
 * @return: success -> OPRT_OK
 */
OPERATE_RET tdd_ws2812_opt_driver_register(IN CHAR_T *driver_name, IN UCHAR_T order_mode)
{
    OPERATE_RET ret = OPRT_OK;
    PIXEL_DRIVER_INTFS_T intfs = {0};
    PIXEL_ATTR_T arrt = {0};

    intfs.open = __tdd_2812_driver_open;
    intfs.output = __tdd_ws2812_driver_send_data;
    intfs.close = __tdd_ws2812_driver_close;

    arrt.color_tp = PIXEL_COLOR_TP_RGB;
    arrt.color_maximum = COLOR_RESOLUTION;

    ret = tdl_pixel_driver_register(driver_name, &intfs, &arrt, NULL);

    rgb_order_mode = order_mode;

    return ret;
}
